-------------------------------------------------------------------------------
--
-- Simple VGA raster display
--
-- Stephen A. Edwards
-- sedwards@cs.columbia.edu
--
-------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity de2_vga_raster is
  
  port (
    reset_n : in std_logic;
    clk   : in std_logic;                    -- Should be 25.125 MHz

    write : in std_logic;------------------
    chipselect : in std_logic;-----------------
    address : in unsigned(6 downto 0);---------------
    writedata : in unsigned(15 downto 0);-------------

    VGA_CLK,                         -- Clock
    VGA_HS,                          -- H_SYNC
    VGA_VS,                          -- V_SYNC
    VGA_BLANK,                       -- BLANK
    VGA_SYNC : out std_logic;        -- SYNC
    VGA_R,                           -- Red[9:0]
    VGA_G,                           -- Green[9:0]
    VGA_B : out unsigned(9 downto 0) -- Blue[9:0]
    );

end de2_vga_raster;

architecture rtl of de2_vga_raster is
  
  -- Video parameters
signal clk25: std_logic;
  constant HTOTAL       : integer := 800;
  constant HSYNC        : integer := 96;
  constant HBACK_PORCH  : integer := 48;
  constant HACTIVE      : integer := 640;
  constant HFRONT_PORCH : integer := 16;
  
  constant VTOTAL       : integer := 525;
  constant VSYNC        : integer := 2;
  constant VBACK_PORCH  : integer := 33;
  constant VACTIVE      : integer := 480;
  constant VFRONT_PORCH : integer := 10;  



--type matrix is array (0 to 63) of integer;
--constant RECTANGLE_HSTART : matrix;
--  RECTANGLE_HSTART[0]<=0;
--
--   FOR  i IN 1 to 64  LOOP
--RECTANGLE_HSTART[i]<=(RECTANGLE[i]+10);
--i=i+1;
--end loop
--
-- 
  constant RECTANGLE_HSTART1 : integer := 1;
  constant RECTANGLE_HEND1   : integer := 8;
  constant RECTANGLE_HSTART2 : integer := 10;
  constant RECTANGLE_HEND2   : integer := 18;
  constant RECTANGLE_HSTART3 : integer := 20;
  constant RECTANGLE_HEND3   : integer := 28;
  constant RECTANGLE_HSTART4 : integer := 30;
  constant RECTANGLE_HEND4   : integer := 38;
  constant RECTANGLE_HSTART5 : integer := 40;
  constant RECTANGLE_HEND5   : integer := 48;
  constant RECTANGLE_HSTART6 : integer := 50;
  constant RECTANGLE_HEND6   : integer := 58;
  constant RECTANGLE_HSTART7 : integer := 60;
  constant RECTANGLE_HEND7   : integer := 68;
  constant RECTANGLE_HSTART8 : integer := 70;
  constant RECTANGLE_HEND8   : integer := 78;
  constant RECTANGLE_HSTART9 : integer := 80;
  constant RECTANGLE_HEND9   : integer := 88;
  constant RECTANGLE_HSTART10 : integer := 90;
  constant RECTANGLE_HEND10   : integer := 98;

  constant RECTANGLE_HSTART11 : integer := 100;
  constant RECTANGLE_HEND11   : integer := 108;
  constant RECTANGLE_HSTART12 : integer := 110;
  constant RECTANGLE_HEND12   : integer := 118;
  constant RECTANGLE_HSTART13 : integer := 120;
  constant RECTANGLE_HEND13   : integer := 128;
  constant RECTANGLE_HSTART14 : integer := 130;
  constant RECTANGLE_HEND14   : integer := 138;
  constant RECTANGLE_HSTART15 : integer := 140;
  constant RECTANGLE_HEND15   : integer := 148;
  constant RECTANGLE_HSTART16 : integer := 150;
  constant RECTANGLE_HEND16   : integer := 158;
  constant RECTANGLE_HSTART17 : integer := 160;
  constant RECTANGLE_HEND17   : integer := 168;
  constant RECTANGLE_HSTART18 : integer := 170;
  constant RECTANGLE_HEND18   : integer := 178;
  constant RECTANGLE_HSTART19 : integer := 180;
  constant RECTANGLE_HEND19   : integer := 188;
  constant RECTANGLE_HSTART20 : integer := 190;
  constant RECTANGLE_HEND20   : integer := 198;



  constant RECTANGLE_HSTART21 : integer := 200;
  constant RECTANGLE_HEND21   : integer := 208;
  constant RECTANGLE_HSTART22 : integer := 210;
  constant RECTANGLE_HEND22   : integer := 218;
  constant RECTANGLE_HSTART23 : integer := 220;
  constant RECTANGLE_HEND23   : integer := 228;
  constant RECTANGLE_HSTART24 : integer := 230;
  constant RECTANGLE_HEND24   : integer := 238;
  constant RECTANGLE_HSTART25 : integer := 240;
  constant RECTANGLE_HEND25   : integer := 248;
  constant RECTANGLE_HSTART26 : integer := 250;
  constant RECTANGLE_HEND26   : integer := 258;
  constant RECTANGLE_HSTART27 : integer := 260;
  constant RECTANGLE_HEND27   : integer := 268;
  constant RECTANGLE_HSTART28 : integer := 270;
  constant RECTANGLE_HEND28   : integer := 278;
  constant RECTANGLE_HSTART29 : integer := 280;
  constant RECTANGLE_HEND29   : integer := 288;
  constant RECTANGLE_HSTART30 : integer := 290;
  constant RECTANGLE_HEND30   : integer := 298;

  constant RECTANGLE_HSTART31 : integer := 300;
  constant RECTANGLE_HEND31   : integer := 308;
  constant RECTANGLE_HSTART32 : integer := 310;
  constant RECTANGLE_HEND32   : integer := 318;
  constant RECTANGLE_HSTART33 : integer := 320;
  constant RECTANGLE_HEND33   : integer := 328;
  constant RECTANGLE_HSTART34 : integer := 330;
  constant RECTANGLE_HEND34   : integer := 338;
  constant RECTANGLE_HSTART35 : integer := 340;
  constant RECTANGLE_HEND35   : integer := 348;
  constant RECTANGLE_HSTART36 : integer := 350;
  constant RECTANGLE_HEND36   : integer := 358;
  constant RECTANGLE_HSTART37 : integer := 360;
  constant RECTANGLE_HEND37   : integer := 368;
  constant RECTANGLE_HSTART38 : integer := 370;
  constant RECTANGLE_HEND38   : integer := 378;
  constant RECTANGLE_HSTART39 : integer := 380;
  constant RECTANGLE_HEND39   : integer := 388;
  constant RECTANGLE_HSTART40 : integer := 390;
  constant RECTANGLE_HEND40   : integer := 398;

  constant RECTANGLE_HSTART41 : integer := 400;
  constant RECTANGLE_HEND41   : integer := 408;
  constant RECTANGLE_HSTART42 : integer := 410;
  constant RECTANGLE_HEND42   : integer := 418;
  constant RECTANGLE_HSTART43 : integer := 420;
  constant RECTANGLE_HEND43   : integer := 428;
  constant RECTANGLE_HSTART44 : integer := 430;
  constant RECTANGLE_HEND44   : integer := 438;
  constant RECTANGLE_HSTART45 : integer := 440;
  constant RECTANGLE_HEND45   : integer := 448;
  constant RECTANGLE_HSTART46 : integer := 450;
  constant RECTANGLE_HEND46   : integer := 458;
  constant RECTANGLE_HSTART47 : integer := 460;
  constant RECTANGLE_HEND47   : integer := 468;
  constant RECTANGLE_HSTART48 : integer := 470;
  constant RECTANGLE_HEND48   : integer := 478;
  constant RECTANGLE_HSTART49 : integer := 480;
  constant RECTANGLE_HEND49   : integer := 488;
  constant RECTANGLE_HSTART50 : integer := 490;
  constant RECTANGLE_HEND50   : integer := 498;


  constant RECTANGLE_HSTART51 : integer := 500;
  constant RECTANGLE_HEND51   : integer := 508;
  constant RECTANGLE_HSTART52 : integer := 510;
  constant RECTANGLE_HEND52   : integer := 518;
  constant RECTANGLE_HSTART53 : integer := 520;
  constant RECTANGLE_HEND53   : integer := 528;
  constant RECTANGLE_HSTART54 : integer := 530;
  constant RECTANGLE_HEND54   : integer := 538;
  constant RECTANGLE_HSTART55 : integer := 540;
  constant RECTANGLE_HEND55   : integer := 548;
  constant RECTANGLE_HSTART56 : integer := 550;
  constant RECTANGLE_HEND56   : integer := 558;
  constant RECTANGLE_HSTART57 : integer := 560;
  constant RECTANGLE_HEND57   : integer := 568;
  constant RECTANGLE_HSTART58 : integer := 570;
  constant RECTANGLE_HEND58   : integer := 578;
  constant RECTANGLE_HSTART59 : integer := 580;
  constant RECTANGLE_HEND59   : integer := 588;
  constant RECTANGLE_HSTART60 : integer := 590;
  constant RECTANGLE_HEND60   : integer := 598;

  constant RECTANGLE_HSTART61 : integer := 600;
  constant RECTANGLE_HEND61   : integer := 608;
  constant RECTANGLE_HSTART62 : integer := 610;
  constant RECTANGLE_HEND62   : integer := 618;
  constant RECTANGLE_HSTART63 : integer := 620;
  constant RECTANGLE_HEND63   : integer := 628;
  constant RECTANGLE_HSTART64 : integer := 630;
  constant RECTANGLE_HEND64   : integer := 638;

  constant RECTANGLE_VEND1,
RECTANGLE_VEND2,
RECTANGLE_VEND3,
RECTANGLE_VEND4, 
RECTANGLE_VEND5,
RECTANGLE_VEND6,
RECTANGLE_VEND7,
RECTANGLE_VEND8,
RECTANGLE_VEND9,  
RECTANGLE_VEND10,
RECTANGLE_VEND11,
RECTANGLE_VEND12,
RECTANGLE_VEND13,
RECTANGLE_VEND14, 
RECTANGLE_VEND15,
RECTANGLE_VEND16,
RECTANGLE_VEND17,
RECTANGLE_VEND18,  
RECTANGLE_VEND19,  
RECTANGLE_VEND20,
RECTANGLE_VEND21,
RECTANGLE_VEND22,
RECTANGLE_VEND23,
RECTANGLE_VEND24, 
RECTANGLE_VEND25,
RECTANGLE_VEND26,
RECTANGLE_VEND27,
RECTANGLE_VEND28,
RECTANGLE_VEND29,  
RECTANGLE_VEND30,
RECTANGLE_VEND31,
RECTANGLE_VEND32,
RECTANGLE_VEND33,
RECTANGLE_VEND34, 
RECTANGLE_VEND35,
RECTANGLE_VEND36,
RECTANGLE_VEND37,
RECTANGLE_VEND38,
RECTANGLE_VEND39,  
RECTANGLE_VEND40,
RECTANGLE_VEND41,
RECTANGLE_VEND42,
RECTANGLE_VEND43,
RECTANGLE_VEND44, 
RECTANGLE_VEND45,
RECTANGLE_VEND46,
RECTANGLE_VEND47,
RECTANGLE_VEND48,
RECTANGLE_VEND49,  
RECTANGLE_VEND50,
RECTANGLE_VEND51,
RECTANGLE_VEND52,
RECTANGLE_VEND53,
RECTANGLE_VEND54, 
RECTANGLE_VEND55,
RECTANGLE_VEND56,
RECTANGLE_VEND57,
RECTANGLE_VEND58,
RECTANGLE_VEND59,  
RECTANGLE_VEND60,
RECTANGLE_VEND61,
RECTANGLE_VEND62,
RECTANGLE_VEND63,
RECTANGLE_VEND64: integer:=380; 


signal RECTANGLE_VSTART1,
RECTANGLE_VSTART2,
RECTANGLE_VSTART3,
RECTANGLE_VSTART4, 
RECTANGLE_VSTART5,
RECTANGLE_VSTART6,
RECTANGLE_VSTART7,
RECTANGLE_VSTART8,
RECTANGLE_VSTART9,  
RECTANGLE_VSTART10,
RECTANGLE_VSTART11,
RECTANGLE_VSTART12,
RECTANGLE_VSTART13,
RECTANGLE_VSTART14, 
RECTANGLE_VSTART15,
RECTANGLE_VSTART16,
RECTANGLE_VSTART17,
RECTANGLE_VSTART18,  
RECTANGLE_VSTART19,  
RECTANGLE_VSTART20,
RECTANGLE_VSTART21,
RECTANGLE_VSTART22,
RECTANGLE_VSTART23,
RECTANGLE_VSTART24, 
RECTANGLE_VSTART25,
RECTANGLE_VSTART26,
RECTANGLE_VSTART27,
RECTANGLE_VSTART28,
RECTANGLE_VSTART29,  
RECTANGLE_VSTART30,
RECTANGLE_VSTART31,
RECTANGLE_VSTART32,
RECTANGLE_VSTART33,
RECTANGLE_VSTART34, 
RECTANGLE_VSTART35,
RECTANGLE_VSTART36,
RECTANGLE_VSTART37,
RECTANGLE_VSTART38,
RECTANGLE_VSTART39,  
RECTANGLE_VSTART40,
RECTANGLE_VSTART41,
RECTANGLE_VSTART42,
RECTANGLE_VSTART43,
RECTANGLE_VSTART44, 
RECTANGLE_VSTART45,
RECTANGLE_VSTART46,
RECTANGLE_VSTART47,
RECTANGLE_VSTART48,
RECTANGLE_VSTART49,  
RECTANGLE_VSTART50,
RECTANGLE_VSTART51,
RECTANGLE_VSTART52,
RECTANGLE_VSTART53,
RECTANGLE_VSTART54, 
RECTANGLE_VSTART55,
RECTANGLE_VSTART56,
RECTANGLE_VSTART57,
RECTANGLE_VSTART58,
RECTANGLE_VSTART59,  
RECTANGLE_VSTART60,
RECTANGLE_VSTART61,
RECTANGLE_VSTART62,
RECTANGLE_VSTART63,
RECTANGLE_VSTART64: unsigned(15 downto 0);

signal vs1:unsigned(15 downto 0);
signal vs2:unsigned(15 downto 0);
signal vs3:unsigned(15 downto 0);
signal vs4:unsigned(15 downto 0);
signal vs5:unsigned(15 downto 0);
signal vs6:unsigned(15 downto 0);
signal vs7:unsigned(15 downto 0);
signal vs8:unsigned(15 downto 0);
signal vs9:unsigned(15 downto 0);
signal vs10:unsigned(15 downto 0);
signal vs11:unsigned(15 downto 0);
signal vs12:unsigned(15 downto 0);
signal vs13:unsigned(15 downto 0);
signal vs14:unsigned(15 downto 0);
signal vs15:unsigned(15 downto 0);
signal vs16:unsigned(15 downto 0);
signal vs17:unsigned(15 downto 0);
signal vs18:unsigned(15 downto 0);
signal vs19:unsigned(15 downto 0);
signal vs20:unsigned(15 downto 0);
signal vs21:unsigned(15 downto 0);
signal vs22:unsigned(15 downto 0);
signal vs23:unsigned(15 downto 0);
signal vs24:unsigned(15 downto 0);
signal vs25:unsigned(15 downto 0);
signal vs26:unsigned(15 downto 0);
signal vs27:unsigned(15 downto 0);
signal vs28:unsigned(15 downto 0);
signal vs29:unsigned(15 downto 0);
signal vs30:unsigned(15 downto 0);
signal vs31:unsigned(15 downto 0);
signal vs32:unsigned(15 downto 0);
signal vs33:unsigned(15 downto 0);
signal vs34:unsigned(15 downto 0);
signal vs35:unsigned(15 downto 0);
signal vs36:unsigned(15 downto 0);
signal vs37:unsigned(15 downto 0);
signal vs38:unsigned(15 downto 0);
signal vs39:unsigned(15 downto 0);
signal vs40:unsigned(15 downto 0);
signal vs41:unsigned(15 downto 0);
signal vs42:unsigned(15 downto 0);
signal vs43:unsigned(15 downto 0);
signal vs44:unsigned(15 downto 0);
signal vs45:unsigned(15 downto 0);
signal vs46:unsigned(15 downto 0);
signal vs47:unsigned(15 downto 0);
signal vs48:unsigned(15 downto 0);
signal vs49:unsigned(15 downto 0);
signal vs50:unsigned(15 downto 0);
signal vs51:unsigned(15 downto 0);
signal vs52:unsigned(15 downto 0);
signal vs53:unsigned(15 downto 0);
signal vs54:unsigned(15 downto 0);
signal vs55:unsigned(15 downto 0);
signal vs56:unsigned(15 downto 0);
signal vs57:unsigned(15 downto 0);
signal vs58:unsigned(15 downto 0);
signal vs59:unsigned(15 downto 0);
signal vs60:unsigned(15 downto 0);
signal vs61:unsigned(15 downto 0);
signal vs62:unsigned(15 downto 0);
signal vs63:unsigned(15 downto 0);
signal vs64:unsigned(15 downto 0);




  -- Signals for the video controller
  signal Hcount : unsigned(9 downto 0);  -- Horizontal position (0-800)
  signal Vcount : unsigned(9 downto 0);  -- Vertical position (0-524)
  signal EndOfLine, EndOfField : std_logic;

  signal vga_hblank, vga_hsync,  vga_vblank, vga_vsync : std_logic;  -- Sync. signals

  signal rectangle_h1, rectangle_v1, rectangle1 : std_logic;  -- rectangle area
  signal rectangle_h2, rectangle_v2, rectangle2 : std_logic;  -- rectangle area
  signal rectangle_h3, rectangle_v3, rectangle3 : std_logic;  -- rectangle area
  signal rectangle_h4, rectangle_v4, rectangle4 : std_logic;  -- rectangle area
  signal rectangle_h5, rectangle_v5, rectangle5 : std_logic;  -- rectangle area
  signal rectangle_h6, rectangle_v6, rectangle6 : std_logic;  -- rectangle area
  signal rectangle_h7, rectangle_v7, rectangle7 : std_logic;  -- rectangle area
  signal rectangle_h8, rectangle_v8, rectangle8 : std_logic;  -- rectangle area
  signal rectangle_h9, rectangle_v9, rectangle9 : std_logic;  -- rectangle area
  signal rectangle_h10, rectangle_v10, rectangle10 : std_logic;  -- rectangle area
  signal rectangle_h11, rectangle_v11, rectangle11 : std_logic;  -- rectangle area
  signal rectangle_h12, rectangle_v12, rectangle12 : std_logic;  -- rectangle area
  signal rectangle_h13, rectangle_v13, rectangle13 : std_logic;  -- rectangle area
  signal rectangle_h14, rectangle_v14, rectangle14 : std_logic;  -- rectangle area
  signal rectangle_h15, rectangle_v15, rectangle15 : std_logic;  -- rectangle area
  signal rectangle_h16, rectangle_v16, rectangle16 : std_logic;  -- rectangle area
  signal rectangle_h17, rectangle_v17, rectangle17 : std_logic;  -- rectangle area
  signal rectangle_h18, rectangle_v18, rectangle18 : std_logic;  -- rectangle area
  signal rectangle_h19, rectangle_v19, rectangle19 : std_logic;  -- rectangle area
  signal rectangle_h20, rectangle_v20, rectangle20 : std_logic;  -- rectangle area
  signal rectangle_h21, rectangle_v21, rectangle21 : std_logic;  -- rectangle area
  signal rectangle_h22, rectangle_v22, rectangle22 : std_logic;  -- rectangle area
  signal rectangle_h23, rectangle_v23, rectangle23 : std_logic;  -- rectangle area
  signal rectangle_h24, rectangle_v24, rectangle24 : std_logic;  -- rectangle area
  signal rectangle_h25, rectangle_v25, rectangle25 : std_logic;  -- rectangle area
  signal rectangle_h26, rectangle_v26, rectangle26 : std_logic;  -- rectangle area
  signal rectangle_h27, rectangle_v27, rectangle27 : std_logic;  -- rectangle area
  signal rectangle_h28, rectangle_v28, rectangle28 : std_logic;  -- rectangle area
  signal rectangle_h29, rectangle_v29, rectangle29 : std_logic;  -- rectangle area
  signal rectangle_h30, rectangle_v30, rectangle30 : std_logic;  -- rectangle area
  signal rectangle_h31, rectangle_v31, rectangle31 : std_logic;  -- rectangle area
  signal rectangle_h32, rectangle_v32, rectangle32 : std_logic;  -- rectangle area
  signal rectangle_h33, rectangle_v33, rectangle33 : std_logic;  -- rectangle area
  signal rectangle_h34, rectangle_v34, rectangle34 : std_logic;  -- rectangle area
  signal rectangle_h35, rectangle_v35, rectangle35 : std_logic;  -- rectangle area
  signal rectangle_h36, rectangle_v36, rectangle36 : std_logic;  -- rectangle area
  signal rectangle_h37, rectangle_v37, rectangle37 : std_logic;  -- rectangle area
  signal rectangle_h38, rectangle_v38, rectangle38 : std_logic;  -- rectangle area
  signal rectangle_h39, rectangle_v39, rectangle39 : std_logic;  -- rectangle area
  signal rectangle_h40, rectangle_v40, rectangle40 : std_logic;  -- rectangle area
  signal rectangle_h41, rectangle_v41, rectangle41 : std_logic;  -- rectangle area
  signal rectangle_h42, rectangle_v42, rectangle42 : std_logic;  -- rectangle area
  signal rectangle_h43, rectangle_v43, rectangle43 : std_logic;  -- rectangle area
  signal rectangle_h44, rectangle_v44, rectangle44 : std_logic;  -- rectangle area
  signal rectangle_h45, rectangle_v45, rectangle45 : std_logic;  -- rectangle area
  signal rectangle_h46, rectangle_v46, rectangle46 : std_logic;  -- rectangle area
  signal rectangle_h47, rectangle_v47, rectangle47 : std_logic;  -- rectangle area
  signal rectangle_h48, rectangle_v48, rectangle48 : std_logic;  -- rectangle area
  signal rectangle_h49, rectangle_v49, rectangle49 : std_logic;  -- rectangle area
  signal rectangle_h50, rectangle_v50, rectangle50 : std_logic;  -- rectangle area
  signal rectangle_h51, rectangle_v51, rectangle51 : std_logic;  -- rectangle area
  signal rectangle_h52, rectangle_v52, rectangle52 : std_logic;  -- rectangle area
  signal rectangle_h53, rectangle_v53, rectangle53 : std_logic;  -- rectangle area
  signal rectangle_h54, rectangle_v54, rectangle54 : std_logic;  -- rectangle area
  signal rectangle_h55, rectangle_v55, rectangle55 : std_logic;  -- rectangle area
  signal rectangle_h56, rectangle_v56, rectangle56 : std_logic;  -- rectangle area
  signal rectangle_h57, rectangle_v57, rectangle57 : std_logic;  -- rectangle area
  signal rectangle_h58, rectangle_v58, rectangle58 : std_logic;  -- rectangle area
  signal rectangle_h59, rectangle_v59, rectangle59 : std_logic;  -- rectangle area
  signal rectangle_h60, rectangle_v60, rectangle60 : std_logic;  -- rectangle area
  signal rectangle_h61, rectangle_v61, rectangle61 : std_logic;  -- rectangle area
  signal rectangle_h62, rectangle_v62, rectangle62 : std_logic;  -- rectangle area
  signal rectangle_h63, rectangle_v63, rectangle63 : std_logic;  -- rectangle area
  signal rectangle_h64, rectangle_v64, rectangle64 : std_logic;  -- rectangle area

begin

clk0:  process (clk)
  begin
    if rising_edge(clk) then
      clk25 <= not clk25;
    end if;
  end process;
 
peripheral: process(clk)
 begin
    if rising_edge(clk) then
      if reset_n = '0' then
        
       vs1<=(others=>'0');
       vs2<=(others=>'0');
       vs3<=(others=>'0');
       vs4<=(others=>'0');
       vs5<=(others=>'0');
       vs6<=(others=>'0');
       vs7<=(others=>'0');
       vs8<=(others=>'0');       
       vs9<=(others=>'0');       
       vs10<=(others=>'0');
       vs11<=(others=>'0');
       vs12<=(others=>'0');
       vs13<=(others=>'0');
       vs14<=(others=>'0');
       vs15<=(others=>'0');
       vs16<=(others=>'0');
       vs17<=(others=>'0');
       vs18<=(others=>'0');       
       vs19<=(others=>'0');       
       vs20<=(others=>'0');
       vs21<=(others=>'0');
       vs22<=(others=>'0');
       vs23<=(others=>'0');
       vs24<=(others=>'0');
       vs25<=(others=>'0');
       vs26<=(others=>'0');
       vs27<=(others=>'0');
       vs28<=(others=>'0');       
       vs29<=(others=>'0');       
       vs30<=(others=>'0');
       vs31<=(others=>'0');
       vs32<=(others=>'0');
       vs33<=(others=>'0');
       vs34<=(others=>'0');
       vs35<=(others=>'0');
       vs36<=(others=>'0');
       vs37<=(others=>'0');
       vs38<=(others=>'0');       
       vs39<=(others=>'0');       
       vs40<=(others=>'0');
       vs41<=(others=>'0');
       vs42<=(others=>'0');
       vs43<=(others=>'0');
       vs44<=(others=>'0');
       vs45<=(others=>'0');
       vs46<=(others=>'0');
       vs47<=(others=>'0');
       vs48<=(others=>'0');       
       vs49<=(others=>'0');       
       vs50<=(others=>'0');
       vs51<=(others=>'0');
       vs52<=(others=>'0');
       vs53<=(others=>'0');
       vs54<=(others=>'0');
       vs55<=(others=>'0');
       vs56<=(others=>'0');
       vs57<=(others=>'0');
       vs58<=(others=>'0');       
       vs59<=(others=>'0');       
       vs60<=(others=>'0');
       vs61<=(others=>'0');
       vs62<=(others=>'0');
       vs63<=(others=>'0');
       vs64<=(others=>'0');
      else
         if chipselect = '1' then
            if write = '1' then
                if address="0000000"
                then   vs1 <= writedata(15 downto 0);
                elsif address="0000001"
                then   vs2<= writedata(15 downto 0);
                elsif address="0000010"
                then   vs3<= writedata(15 downto 0);
                elsif address="0000011"
                then   vs4<= writedata(15 downto 0);
                elsif address="0000100"
                then   vs5<= writedata(15 downto 0);
                elsif address="0000101"
                then   vs6<= writedata(15 downto 0);
                elsif address="0000110"
                then   vs7<= writedata(15 downto 0);
                elsif address="0000111"
                then   vs8<= writedata(15 downto 0);
                elsif address="0001000"
                  then   vs9 <= writedata(15 downto 0);
                elsif address="0001001"
                  then   vs10<= writedata(15 downto 0);
                elsif address="0001010"
                  then   vs11<= writedata(15 downto 0);
                elsif address="0001011"
                  then   vs12<= writedata(15 downto 0);
                elsif address="0001100"
                  then   vs13<= writedata(15 downto 0);
                elsif address="0001101"
                  then   vs14<= writedata(15 downto 0);
                elsif address="0001110"
                  then   vs15<= writedata(15 downto 0);
                elsif address="0001111"
                  then   vs16<= writedata(15 downto 0);

                elsif address="0010000"
                then   vs17 <= writedata(15 downto 0);
                elsif address="0010001"
                then   vs18<= writedata(15 downto 0);
                elsif address="0010010"
                then   vs19<= writedata(15 downto 0);
                elsif address="0010011"
                then   vs20<= writedata(15 downto 0);
                elsif address="0010100"
                then   vs21<= writedata(15 downto 0);
                elsif address="0010101"
                then   vs22<= writedata(15 downto 0);
                elsif address="0010110"
                then   vs23<= writedata(15 downto 0);
                elsif address="0010111"
                then   vs24<= writedata(15 downto 0);
                elsif address="0011000"
                  then   vs25 <= writedata(15 downto 0);
                elsif address="0011001"
                  then   vs26<= writedata(15 downto 0);

                elsif address="0011010"
                then   vs27 <= writedata(15 downto 0);
                elsif address="0011011"
                then   vs28<= writedata(15 downto 0);
                elsif address="0011100"
                then   vs29<= writedata(15 downto 0);
                elsif address="0011101"
                then   vs30<= writedata(15 downto 0);
                elsif address="0011110"
                then   vs31<= writedata(15 downto 0);
                elsif address="0011111"
                then   vs32<= writedata(15 downto 0);
                elsif address="0100000"
                then   vs33<= writedata(15 downto 0);
                elsif address="0100001"
                then   vs34<= writedata(15 downto 0);
                elsif address="0100010"
                  then   vs35 <= writedata(15 downto 0);
                elsif address="0100011"
                  then   vs36<= writedata(15 downto 0);

                elsif address="0100100"
                then   vs37 <= writedata(15 downto 0);
                elsif address="0100101"
                then   vs38<= writedata(15 downto 0);
                elsif address="0100110"
                then   vs39<= writedata(15 downto 0);
                elsif address="0100111"
                then   vs40<= writedata(15 downto 0);
                elsif address="0101000"
                then   vs41<= writedata(15 downto 0);
               elsif address="0101001"
                then   vs42<= writedata(15 downto 0);
                elsif address="0101010"
                then   vs43<= writedata(15 downto 0);
                elsif address="0101011"
                then   vs44<= writedata(15 downto 0);
                elsif address="0101100"
                  then   vs45 <= writedata(15 downto 0);
                elsif address="0101101"
                  then   vs46<= writedata(15 downto 0);

                elsif address="0101110"
                 then  vs47 <= writedata(15 downto 0);
                elsif address="0101111"
                then   vs48<= writedata(15 downto 0);
                elsif address="0110000"
                then   vs49<= writedata(15 downto 0);
                elsif address="0110001"
                then   vs50<= writedata(15 downto 0);
                elsif address="0110010"
                then   vs51<= writedata(15 downto 0);
                elsif address="0110011"
                then   vs52<= writedata(15 downto 0);
                elsif address="0110100"
                then   vs53<= writedata(15 downto 0);
                elsif address="0110101"
                then   vs54<= writedata(15 downto 0);
                elsif address="0110110"
                then  vs55 <= writedata(15 downto 0);
          elsif address="0110111"
                  then   vs56<= writedata(15 downto 0);

               elsif address="0111000"
                 then  vs57 <= writedata(15 downto 0);
           elsif address="0111001"
                then   vs58<= writedata(15 downto 0);
             elsif address="0111010"
                then   vs59<= writedata(15 downto 0);
       elsif address="0111011"
                then   vs60<= writedata(15 downto 0);
                elsif address="0111100"
                then   vs61<= writedata(15 downto 0);
              elsif address="0111101"
                then   vs62<= writedata(15 downto 0);
                elsif address="0111110"
                then   vs63<= writedata(15 downto 0);
                elsif address="0111111"
                then   vs64<= writedata(15 downto 0);
                end if;

              
            end if;
         end if;
         if EndOfLine='1' and EndOfField='1'
         then RECTANGLE_VSTART1<=vs1;
              RECTANGLE_VSTART2<=vs2;
              RECTANGLE_VSTART3<=vs3;
              RECTANGLE_VSTART4<=vs4;
              RECTANGLE_VSTART5<=vs5;
              RECTANGLE_VSTART6<=vs6;
              RECTANGLE_VSTART7<=vs7;
              RECTANGLE_VSTART8<=vs8;
              RECTANGLE_VSTART9<=vs9;
              RECTANGLE_VSTART10<=vs10;
              RECTANGLE_VSTART11<=vs11;
              RECTANGLE_VSTART12<=vs12;
              RECTANGLE_VSTART13<=vs13;
              RECTANGLE_VSTART14<=vs14;
              RECTANGLE_VSTART15<=vs15;
              RECTANGLE_VSTART16<=vs16;
              
              RECTANGLE_VSTART17<=vs17;
              RECTANGLE_VSTART18<=vs18;
              RECTANGLE_VSTART19<=vs19;
              RECTANGLE_VSTART20<=vs20;
              RECTANGLE_VSTART21<=vs21;
              RECTANGLE_VSTART22<=vs22;
              RECTANGLE_VSTART23<=vs23;
              RECTANGLE_VSTART24<=vs24;
              RECTANGLE_VSTART25<=vs25;
              RECTANGLE_VSTART26<=vs26;
              RECTANGLE_VSTART27<=vs27;
              RECTANGLE_VSTART28<=vs28;
              RECTANGLE_VSTART29<=vs29;
              RECTANGLE_VSTART30<=vs30;

              RECTANGLE_VSTART31<=vs31;
              RECTANGLE_VSTART32<=vs32;
              RECTANGLE_VSTART33<=vs33;
              RECTANGLE_VSTART34<=vs34;
              RECTANGLE_VSTART35<=vs35;
              RECTANGLE_VSTART36<=vs36;
              RECTANGLE_VSTART37<=vs37;
              RECTANGLE_VSTART38<=vs38;
              RECTANGLE_VSTART39<=vs39;
              RECTANGLE_VSTART40<=vs40;

              RECTANGLE_VSTART41<=vs41;
              RECTANGLE_VSTART42<=vs42;
              RECTANGLE_VSTART43<=vs43;
              RECTANGLE_VSTART44<=vs44;
              RECTANGLE_VSTART45<=vs45;
              RECTANGLE_VSTART46<=vs46;
              RECTANGLE_VSTART47<=vs47;
              RECTANGLE_VSTART48<=vs48;
              RECTANGLE_VSTART49<=vs49;
              RECTANGLE_VSTART50<=vs50;

              RECTANGLE_VSTART51<=vs51;
              RECTANGLE_VSTART52<=vs52;
              RECTANGLE_VSTART53<=vs53;
              RECTANGLE_VSTART54<=vs54;
              RECTANGLE_VSTART55<=vs55;
              RECTANGLE_VSTART56<=vs56;
              RECTANGLE_VSTART57<=vs57;
              RECTANGLE_VSTART58<=vs58;
              RECTANGLE_VSTART59<=vs59;
              RECTANGLE_VSTART60<=vs60;
              RECTANGLE_VSTART61<=vs61;
              RECTANGLE_VSTART62<=vs62;
              RECTANGLE_VSTART63<=vs63;
              RECTANGLE_VSTART64<=vs64;

         end if;
       end if;
    end if;
 end process peripheral;







  -- Horizontal and vertical counters

  HCounter : process (clk25)
  begin
    if rising_edge(clk25) then      
      if reset_n = '0' then
        Hcount <= (others => '0');
      elsif EndOfLine = '1' then
        Hcount <= (others => '0');
      else
        Hcount <= Hcount + 1;
      end if;      
    end if;
  end process HCounter;

  EndOfLine <= '1' when Hcount = HTOTAL - 1 else '0';
  
  VCounter: process (clk25)
  begin
    if rising_edge(clk25) then      
      if reset_n = '0' then
        Vcount <= (others => '0');
      elsif EndOfLine = '1' then
        if EndOfField = '1' then
          Vcount <= (others => '0');
        else
          Vcount <= Vcount + 1;
        end if;
      end if;
    end if;
  end process VCounter;

  EndOfField <= '1' when Vcount = VTOTAL - 1 else '0';

  -- State machines to generate HSYNC, VSYNC, HBLANK, and VBLANK

  HSyncGen : process (clk25)
  begin
    if rising_edge(clk25) then     
      if reset_n = '0' or EndOfLine = '1' then
        vga_hsync <= '1';
      elsif Hcount = HSYNC - 1 then
        vga_hsync <= '0';
      end if;
    end if;
  end process HSyncGen;
  
  HBlankGen : process (clk25)
  begin
    if rising_edge(clk25) then
      if reset_n = '0' then
        vga_hblank <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH then
        vga_hblank <= '0';
      elsif Hcount = HSYNC + HBACK_PORCH + HACTIVE then
        vga_hblank <= '1';
      end if;      
    end if;
  end process HBlankGen;

  VSyncGen : process (clk25)
  begin
    if rising_edge(clk25) then
      if reset_n = '0' then
        vga_vsync <= '1';
      elsif EndOfLine ='1' then
        if EndOfField = '1' then
          vga_vsync <= '1';
        elsif Vcount = VSYNC - 1 then
          vga_vsync <= '0';
        end if;
      end if;      
    end if;
  end process VSyncGen;

  VBlankGen : process (clk25)
  begin
    if rising_edge(clk25) then    
      if reset_n = '0' then
        vga_vblank <= '1';
      elsif EndOfLine = '1' then
        if Vcount = VSYNC + VBACK_PORCH - 1 then
          vga_vblank <= '0';
        elsif Vcount = VSYNC + VBACK_PORCH + VACTIVE - 1 then
          vga_vblank <= '1';
        end if;
      end if;
    end if;
  end process VBlankGen;

  -- Rectangle generator

  RectangleHGen : process (clk25)
  begin
    if rising_edge(clk25) then    

      if reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART1) then
        rectangle_h1 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND1 then
        rectangle_h1 <= '0'; 
      end if;
      
      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART2) then
        rectangle_h2 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND2 then
        rectangle_h2 <= '0'; 
      end if;
      
      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART3) then
        rectangle_h3 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND3 then
        rectangle_h3 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART4) then
        rectangle_h4 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND4 then
        rectangle_h4 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART5) then
        rectangle_h5 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND5 then
        rectangle_h5 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART6) then
        rectangle_h6 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND6 then
        rectangle_h6 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART7) then
        rectangle_h7 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND7 then
        rectangle_h7 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART8) then
        rectangle_h8 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND8 then
        rectangle_h8 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART9) then
        rectangle_h9 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND9 then
        rectangle_h9 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART10) then
        rectangle_h10 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND10 then
        rectangle_h10 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART11) then
        rectangle_h11 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND11 then
        rectangle_h11 <= '0'; 
      end if;
      
      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART12) then
        rectangle_h12 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND12 then
        rectangle_h12 <= '0'; 
      end if;
      
      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART13) then
        rectangle_h13 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND13 then
        rectangle_h13 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART14) then
        rectangle_h14 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND14 then
        rectangle_h14 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART15) then
        rectangle_h15 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND15 then
        rectangle_h15 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART16) then
        rectangle_h16 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND16 then
        rectangle_h16 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART17) then
        rectangle_h17 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND17 then
        rectangle_h17 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART18) then
        rectangle_h18 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND18 then
        rectangle_h18 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART19) then
        rectangle_h19 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND19 then
        rectangle_h19 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART20) then
        rectangle_h20 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND20 then
        rectangle_h20 <= '0'; 
      end if;


      if  reset_n = '0' or  (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART21) then
        rectangle_h21 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND21 then
        rectangle_h21 <= '0'; 
      end if;
      
      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART22) then
        rectangle_h22 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND22 then
        rectangle_h22 <= '0'; 
      end if;
      
      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART23) then
        rectangle_h23 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND23 then
        rectangle_h23 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART24) then
        rectangle_h24 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND24 then
        rectangle_h24 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART25) then
        rectangle_h25 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND25 then
        rectangle_h25 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART26) then
        rectangle_h26 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND26 then
        rectangle_h26 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART27) then
        rectangle_h27 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND27 then
        rectangle_h27 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART28) then
        rectangle_h28 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND28 then
        rectangle_h28 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART29) then
        rectangle_h29 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND29 then
        rectangle_h29 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART30) then
        rectangle_h30 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND30 then
        rectangle_h30 <= '0'; 
      end if;

      if  reset_n = '0' or  (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART31) then
        rectangle_h31 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND31 then
        rectangle_h31 <= '0'; 
      end if;
      
      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART32) then
        rectangle_h32 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND32 then
        rectangle_h32 <= '0'; 
      end if;
      
      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART33) then
        rectangle_h33 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND33 then
        rectangle_h33 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART34) then
        rectangle_h34 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND34 then
        rectangle_h34 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART35) then
        rectangle_h35 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND35 then
        rectangle_h35 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART36) then
        rectangle_h36 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND36 then
        rectangle_h36 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART37) then
        rectangle_h37 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND37 then
        rectangle_h37 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART38) then
        rectangle_h38 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND38 then
        rectangle_h38 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART39) then
        rectangle_h39 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND39 then
        rectangle_h39 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART40) then
        rectangle_h40 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND40 then
        rectangle_h40 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART41) then
        rectangle_h41 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND41 then
        rectangle_h41 <= '0'; 
      end if;
      
      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART42) then
        rectangle_h42 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND42 then
        rectangle_h42 <= '0'; 
      end if;
      
      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART43) then
        rectangle_h43 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND43 then
        rectangle_h43 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART44) then
        rectangle_h44 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND44 then
        rectangle_h44 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART45) then
        rectangle_h45 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND45 then
        rectangle_h45 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART46) then
        rectangle_h46 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND46 then
        rectangle_h46 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART47) then
        rectangle_h47 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND47 then
        rectangle_h47 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART48) then
        rectangle_h48 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND48 then
        rectangle_h48 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART49) then
        rectangle_h49 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND49 then
        rectangle_h49 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART50) then
        rectangle_h50 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND50 then
        rectangle_h50 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART51) then
        rectangle_h51 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND51 then
        rectangle_h51 <= '0'; 
      end if;
      
      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART52) then
        rectangle_h52 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND52 then
        rectangle_h52 <= '0'; 
      end if;
      
      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART53) then
        rectangle_h53 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND53 then
        rectangle_h53 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART54) then
        rectangle_h54 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND54 then
        rectangle_h54 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART55) then
        rectangle_h55 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND55 then
        rectangle_h55 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART56) then
        rectangle_h56 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND56 then
        rectangle_h56 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART57) then
        rectangle_h57 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND57 then
        rectangle_h57 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART58) then
        rectangle_h58 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND58 then
        rectangle_h58 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART59) then
        rectangle_h59 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND59 then
        rectangle_h59 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART60) then
        rectangle_h60 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND60 then
        rectangle_h60 <= '0'; 
      end if;



      if   reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART61) then
        rectangle_h61 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND61 then
        rectangle_h61 <= '0'; 
      end if;
      
      if reset_n = '0' or  (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART62) then
        rectangle_h62 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND62 then
        rectangle_h62 <= '0'; 
      end if;
      
      if reset_n = '0' or  (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART63) then
        rectangle_h63 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND63 then
        rectangle_h63 <= '0'; 
      end if;

      if  reset_n = '0' or (Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HSTART64) then
        rectangle_h64 <= '1';
      elsif Hcount = HSYNC + HBACK_PORCH + RECTANGLE_HEND64 then
        rectangle_h64 <= '0'; 
      end if;

  end if;

  end process RectangleHGen;



  RectangleVGen : process (clk25)
  begin
    if rising_edge(clk25) then
      if reset_n = '0' then       
        rectangle_v1 <= '0'; 
        rectangle_v2 <= '0'; 
        rectangle_v3 <= '0';
        rectangle_v4 <= '0'; 
        rectangle_v5 <= '0'; 
        rectangle_v6 <= '0';
        rectangle_v7 <= '0'; 
        rectangle_v8 <= '0'; 
        rectangle_v9 <= '0';
        rectangle_v10 <= '0';     
        rectangle_v11 <= '0'; 
        rectangle_v12 <= '0'; 
        rectangle_v13 <= '0';
        rectangle_v14 <= '0'; 
        rectangle_v15 <= '0'; 
        rectangle_v16 <= '0';
        rectangle_v17 <= '0'; 
        rectangle_v18 <= '0'; 
        rectangle_v19 <= '0';
        rectangle_v20 <= '0';  
        rectangle_v21 <= '0'; 
        rectangle_v22 <= '0'; 
        rectangle_v23 <= '0';
        rectangle_v24 <= '0'; 
        rectangle_v25 <= '0'; 
        rectangle_v26 <= '0';
        rectangle_v27 <= '0'; 
        rectangle_v28 <= '0'; 
        rectangle_v29 <= '0';
        rectangle_v30 <= '0';    
        rectangle_v31 <= '0'; 
        rectangle_v32 <= '0'; 
        rectangle_v33 <= '0';
        rectangle_v34 <= '0'; 
        rectangle_v35 <= '0'; 
        rectangle_v36 <= '0';
        rectangle_v37 <= '0'; 
        rectangle_v38 <= '0'; 
        rectangle_v39 <= '0';
        rectangle_v40 <= '0';
        rectangle_v41 <= '0'; 
        rectangle_v42 <= '0'; 
        rectangle_v43 <= '0';
        rectangle_v44 <= '0'; 
        rectangle_v45 <= '0'; 
        rectangle_v46 <= '0';
        rectangle_v47 <= '0'; 
        rectangle_v48 <= '0'; 
        rectangle_v49 <= '0';
        rectangle_v50 <= '0';
        rectangle_v51 <= '0'; 
        rectangle_v52 <= '0'; 
        rectangle_v53 <= '0';
        rectangle_v54 <= '0'; 
        rectangle_v55 <= '0'; 
        rectangle_v56 <= '0';
        rectangle_v57 <= '0'; 
        rectangle_v58 <= '0'; 
        rectangle_v59 <= '0';
        rectangle_v60 <= '0';
        rectangle_v61 <= '0'; 
        rectangle_v62 <= '0'; 
        rectangle_v63 <= '0';
        rectangle_v64 <= '0'; 

 

      elsif EndOfLine = '1' then

      if  Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART1 then
        rectangle_v1 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND1 then
        rectangle_v1 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART2) then
        rectangle_v2 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND2 then
        rectangle_v2 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART3) then
        rectangle_v3 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND3 then
        rectangle_v3 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART4) then
        rectangle_v4 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND4 then
        rectangle_v4 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART5) then
        rectangle_v5 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND5 then
        rectangle_v5 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART6) then
        rectangle_v6 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND6 then
        rectangle_v6 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART7) then
        rectangle_v7 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND7 then
        rectangle_v7 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART8) then
        rectangle_v8 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND8 then
        rectangle_v8 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART9) then
        rectangle_v9 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND9 then
        rectangle_v9 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART10) then
        rectangle_v10 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND10 then
        rectangle_v10 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART11) then
        rectangle_v11 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND11 then
        rectangle_v11 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART12) then
        rectangle_v12 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND12 then
        rectangle_v12 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART13) then
        rectangle_v13 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND13 then
        rectangle_v13 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART14) then
        rectangle_v14 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND14 then
        rectangle_v14 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART15) then
        rectangle_v15 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND15 then
        rectangle_v15 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART16) then
        rectangle_v16 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND16 then
        rectangle_v16 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART17) then
        rectangle_v17 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND17 then
        rectangle_v17 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART18) then
        rectangle_v18 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND18 then
        rectangle_v18 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART19) then
        rectangle_v19 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND19 then
        rectangle_v19 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART20) then
        rectangle_v20 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND20 then
        rectangle_v20 <= '0';
      end if;


      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART21) then
        rectangle_v21 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND21 then
        rectangle_v21 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART22) then
        rectangle_v22 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND22 then
        rectangle_v22 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART23) then
        rectangle_v23 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND23 then
        rectangle_v23 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART24) then
        rectangle_v24 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND24 then
        rectangle_v24 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART25) then
        rectangle_v25 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND25 then
        rectangle_v25 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART26) then
        rectangle_v26 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND26 then
        rectangle_v26 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART27) then
        rectangle_v27 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND27 then
        rectangle_v27 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART28) then
        rectangle_v28 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND28 then
        rectangle_v28 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART29) then
        rectangle_v29 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND29 then
        rectangle_v29 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART30) then
        rectangle_v30 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND30 then
        rectangle_v30 <= '0';
      end if;

      if  (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART31) then
        rectangle_v31 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND31 then
        rectangle_v31 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART32) then
        rectangle_v32 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND32 then
        rectangle_v32 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART33) then
        rectangle_v33 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND33 then
        rectangle_v33 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART34) then
        rectangle_v34 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND34 then
        rectangle_v34 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART35) then
        rectangle_v35 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND35 then
        rectangle_v35 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART36) then
        rectangle_v36 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND36 then
        rectangle_v36 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART37) then
        rectangle_v37 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND37 then
        rectangle_v37 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART38) then
        rectangle_v38 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND38 then
        rectangle_v38 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART39) then
        rectangle_v39 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND39 then
        rectangle_v39 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART40) then
        rectangle_v40 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND40 then
        rectangle_v40 <= '0';
      end if;

      if(Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART41) then
        rectangle_v41 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND41 then
        rectangle_v41 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART42) then
        rectangle_v42 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND42 then
        rectangle_v42 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART43) then
        rectangle_v43 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND43 then
        rectangle_v43 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART44) then
        rectangle_v44 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND44 then
        rectangle_v44 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART45) then
        rectangle_v45 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND45 then
        rectangle_v45 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART46) then
        rectangle_v46 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND46 then
        rectangle_v46 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART47) then
        rectangle_v47 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND47 then
        rectangle_v47 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART48) then
        rectangle_v48 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND48 then
        rectangle_v48 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART49) then
        rectangle_v49 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND49 then
        rectangle_v49 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART50) then
        rectangle_v50 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND50 then
        rectangle_v50 <= '0';
      end if;

      if  (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART51) then
        rectangle_v51 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND51 then
        rectangle_v51 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART52) then
        rectangle_v52 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND52 then
        rectangle_v52 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART53) then
        rectangle_v53 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND53 then
        rectangle_v53 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART54) then
        rectangle_v54 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND54 then
        rectangle_v54 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART55) then
        rectangle_v55 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND55 then
        rectangle_v55 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART56) then
        rectangle_v56 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND56 then
        rectangle_v56 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART57) then
        rectangle_v57 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND57 then
        rectangle_v57 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART58) then
        rectangle_v58 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND58 then
        rectangle_v58 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART59) then
        rectangle_v59 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND59 then
        rectangle_v59 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART60) then
        rectangle_v60 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND60 then
        rectangle_v60 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART61) then
        rectangle_v61 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND61 then
        rectangle_v61 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART62) then
        rectangle_v62 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND62 then
        rectangle_v62 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART63) then
        rectangle_v63 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND63 then
        rectangle_v63 <= '0';
      end if;

      if (Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VSTART64) then
        rectangle_v64 <= '1';
      elsif Vcount = VSYNC + VBACK_PORCH-1 + RECTANGLE_VEND64 then
        rectangle_v64 <= '0';
      end if;

    end if;      
   end if;
 end process RectangleVGen;


  rectangle1 <= rectangle_h1 and rectangle_v1;
  rectangle2 <= rectangle_h2 and rectangle_v2;
  rectangle3 <= rectangle_h3 and rectangle_v3;
  rectangle4 <= rectangle_h4 and rectangle_v4;
  rectangle5 <= rectangle_h5 and rectangle_v5;
  rectangle6 <= rectangle_h6 and rectangle_v6;
  rectangle7 <= rectangle_h7 and rectangle_v7;
  rectangle8 <= rectangle_h8 and rectangle_v8;
  rectangle9 <= rectangle_h9 and rectangle_v9;
  rectangle10 <= rectangle_h10 and rectangle_v10;
  rectangle11 <= rectangle_h11 and rectangle_v11;
  rectangle12 <= rectangle_h12 and rectangle_v12;
  rectangle13 <= rectangle_h13 and rectangle_v13;
  rectangle14 <= rectangle_h14 and rectangle_v14;
  rectangle15 <= rectangle_h15 and rectangle_v15;
  rectangle16 <= rectangle_h16 and rectangle_v16;
  rectangle17 <= rectangle_h17 and rectangle_v17;
  rectangle18 <= rectangle_h18 and rectangle_v18;
  rectangle19 <= rectangle_h19 and rectangle_v19;
  rectangle20 <= rectangle_h20 and rectangle_v20;
  rectangle21 <= rectangle_h21 and rectangle_v21;
  rectangle22 <= rectangle_h22 and rectangle_v22;
  rectangle23 <= rectangle_h23 and rectangle_v23;
  rectangle24 <= rectangle_h24 and rectangle_v24;
  rectangle25 <= rectangle_h25 and rectangle_v25;
  rectangle26 <= rectangle_h26 and rectangle_v26;
  rectangle27 <= rectangle_h27 and rectangle_v27;
  rectangle28 <= rectangle_h28 and rectangle_v28;
  rectangle29 <= rectangle_h29 and rectangle_v29;
  rectangle30 <= rectangle_h30 and rectangle_v30;
  rectangle31 <= rectangle_h31 and rectangle_v31;
  rectangle32 <= rectangle_h32 and rectangle_v32;
  rectangle33 <= rectangle_h33 and rectangle_v33;
  rectangle34 <= rectangle_h34 and rectangle_v34;
  rectangle35 <= rectangle_h35 and rectangle_v35;
  rectangle36 <= rectangle_h36 and rectangle_v36;
  rectangle37 <= rectangle_h37 and rectangle_v37;
  rectangle38 <= rectangle_h38 and rectangle_v38;
  rectangle39 <= rectangle_h39 and rectangle_v39;
  rectangle40 <= rectangle_h40 and rectangle_v40;
  rectangle41 <= rectangle_h41 and rectangle_v41;
  rectangle42 <= rectangle_h42 and rectangle_v42;
  rectangle43 <= rectangle_h43 and rectangle_v43;
  rectangle44 <= rectangle_h44 and rectangle_v44;
  rectangle45 <= rectangle_h45 and rectangle_v45;
  rectangle46 <= rectangle_h46 and rectangle_v46;
  rectangle47 <= rectangle_h47 and rectangle_v47;
  rectangle48 <= rectangle_h48 and rectangle_v48;
  rectangle49 <= rectangle_h49 and rectangle_v49;
  rectangle50 <= rectangle_h50 and rectangle_v50;
  rectangle51 <= rectangle_h51 and rectangle_v51;
  rectangle52 <= rectangle_h52 and rectangle_v52;
  rectangle53 <= rectangle_h53 and rectangle_v53;
  rectangle54 <= rectangle_h54 and rectangle_v54;
  rectangle55 <= rectangle_h55 and rectangle_v55;
  rectangle56 <= rectangle_h56 and rectangle_v56;
  rectangle57 <= rectangle_h57 and rectangle_v57;
  rectangle58 <= rectangle_h58 and rectangle_v58;
  rectangle59 <= rectangle_h59 and rectangle_v59;
  rectangle60 <= rectangle_h60 and rectangle_v60;
  rectangle61 <= rectangle_h61 and rectangle_v61;
  rectangle62 <= rectangle_h62 and rectangle_v62;
  rectangle63 <= rectangle_h63 and rectangle_v63;
  rectangle64 <= rectangle_h64 and rectangle_v64;
  -- Registered video signals going to the video DAC

  VideoOut: process (clk25, reset_n)
  begin
    if reset_n = '0' then
      VGA_R <= "0000000000";
      VGA_G <= "0000000000";
      VGA_B <= "0000000000";
    elsif clk25'event and clk25 = '1' then

if (rectangle1 ='1'or rectangle2 ='1'or rectangle3 ='1'or rectangle4 ='1'or rectangle5 ='1'
or rectangle6 ='1'or rectangle7 ='1'or  rectangle8 ='1'or  rectangle9 ='1'
or  rectangle10 ='1'or rectangle11 ='1'or rectangle12 ='1'or  rectangle13 ='1'
or rectangle14 ='1'or rectangle15 ='1'or  rectangle16 ='1'or  rectangle17 ='1'
or  rectangle18 ='1'or  rectangle19 ='1'or  rectangle20 ='1'or  rectangle21 ='1'
or  rectangle22 ='1'or  rectangle23 ='1'or  rectangle24 ='1'or  rectangle25 ='1'
or rectangle26 ='1'or rectangle27 ='1'or  rectangle28 ='1'or  rectangle29 ='1'
or  rectangle30 ='1'or  rectangle31 ='1'or  rectangle32 ='1'or  rectangle33 ='1'
or  rectangle34 ='1'or  rectangle35 ='1'or  rectangle36 ='1'or  rectangle37 ='1'
or  rectangle38 ='1'or rectangle39 ='1'or  rectangle40 ='1'or  rectangle41 ='1'
or  rectangle42 ='1'or  rectangle43 ='1'or  rectangle44 ='1'or rectangle45 ='1'
or  rectangle46 ='1'or  rectangle47 ='1'or rectangle48 ='1'or rectangle49 ='1'
or rectangle50 ='1'or rectangle51 ='1' or  rectangle52 ='1'or  rectangle53 ='1'
or  rectangle54 ='1'or  rectangle55 ='1'or  rectangle56 ='1'or  rectangle57 ='1'
or  rectangle58 ='1'or rectangle59 ='1'or  rectangle60 ='1'
or rectangle61 ='1'or rectangle62 ='1'
or  rectangle63 ='1'or  rectangle64 ='1') then

        VGA_R <= "1111111111";
        VGA_G <= "1111111111";
        VGA_B <= "1111111111";


      elsif vga_hblank = '0' and vga_vblank ='0' then
        VGA_R <= "0000000000";
        VGA_G <= "0000000000";
        VGA_B <= "0000000000";
      else
        VGA_R <= "0000000000";
        VGA_G <= "0000000000";
        VGA_B <= "0000000000";    
      end if;
    end if;
  end process VideoOut;

 

  VGA_CLK <= clk25;
  VGA_HS <= not vga_hsync;
  VGA_VS <= not vga_vsync;
  VGA_SYNC <= '0';
  VGA_BLANK <= not (vga_hsync or vga_vsync);

end rtl;


---------------------------


